#
# http.pl -- common definitions and functions for HTTP handling routine.
# Copyright (C) 1996 nir@mvg.biglobe.ne.jp
#
# This is free script; you can redistribute it and/or modify it
# for any purpose.
#
# @(#)$http.pl,v2.13 30 Jan 1997 nir $
#

package http;

@wday = ('Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat');
@weekday = (
 'Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday');
@mon = (
 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');
@month = (
 'January', 'February', 'March',     'April',   'May',      'June',
 'July',    'August',   'September', 'October', 'November', 'December');

$server_adj = 0;
$local_adj  = 0;

sub move_page {
	local($_) = @_;

	printf("Location: %s%s\n\n", &server_url, $_);
	exit(0);
}

sub server_url {
	local($host, $name, $port);

	$host = $ENV{'HTTP_HOST'};
	if ($host ne '') {
		sprintf("http://%s", $host);
	} else {
		$name = $ENV{'SERVER_NAME'};
		$port = $ENV{'SERVER_PORT'};
		if ($port == 80) {
			sprintf("http://%s", $name);
		} else {
			sprintf("http://%s:%d", $name, $port);
		}
	}
}

sub url_decode {
	local($n);
	local(@line);
	local($_) = @_;

	@line = split(/%0[aA]/);
	for ($n = 0; $n <= $#line; $n++) {
		$line[$n] =~ tr/+/ /;
		$line[$n] =~ s/%([a-f0-9]{2})/pack('C', hex($1))/ieg;
	}
	return(join("\x0A", @line));
}

sub url_encode {
	local($_) = @_;

	s#([^/_!\$\-\.\*\(\)\w])#'%'.unpack('H2', $1)#eg;
	s/%20/+/g;
	return($_);
}

sub http_args {
	&conv_http_args(&get_http_args);
}

sub get_http_args {
	local($args, $len, $query);

	$args = '';
	if ($ENV{'REQUEST_METHOD'} eq 'POST') {
		$len = $ENV{'CONTENT_LENGTH'} + 0;
		read(STDIN, $args, $len);
	}
	$query = $ENV{'QUERY_STRING'};
	if ($query ne '') {
		$args .= '&' if ($args ne '');
		$args .= $query;
	}
	return($args);
}

sub conv_http_args {
	local($tag, $body);
	local(%http_args);
	local($args) = @_;

	foreach $_ (split(/&/, $args)) {
		($tag, $body) = split(/=/);
		$http_args{&url_decode($tag)} = &url_decode($body);
	}
	return(%http_args);
}

sub make_http_args {
	local($key);
	local(@args);
	local(%http_args) = @_;

	foreach $key (keys %http_args) {
		push(@args, join('=', &url_encode($key), &url_encode($http_args{$key})));
	}
	return(join('&', @args));
}

sub http_cookie {
	local($name, $value);
	local(%http_cookie);

	local($cookie) = $ENV{'HTTP_COOKIE'};
	foreach $_ (split(/; /, $cookie)) {
		($name, $value) = split(/=/);
		$name = &url_decode($name);
		$http_cookie{$name} = &url_decode($value)
			if ($http_cookie{$name} eq '');
	}
	return(%http_cookie);
}

sub set_cookie {
	local($output);
	local($name, $value, $expires, $path, $domain, $secure) = @_;

	return('') if ($name eq '');
	$output = sprintf("%s=%s", &url_encode($name), &url_encode($value));
	$output .= sprintf("; expires=%s", &http_date($expires)) if ($expires);
	$output .= "; domain=$domain" if ($domain ne '');
	$output .= "; path=$path" if ($path ne '');
	$output .= "; secure" if ($secure);
	return("Set-Cookie: $output");
}

sub http_date {
	local($sec, $min, $hour, $mday, $mon, $year, $wday);
	local($time) = @_;

	($sec, $min, $hour, $mday, $mon, $year, $wday) = gmtime($time - $server_adj);
	sprintf("%s, %02d %s %04d %02d:%02d:%02d GMT",
	$wday[$wday], $mday, $mon[$mon], $year + 1900, $hour, $min, $sec);
}

sub local_time {
	localtime($_[0] + $local_adj);
}

sub http_quote {
	local($_) = @_;

	s/&/&amp;/g;
	s/</&lt;/g;
	s/>/&gt;/g;
	s/"/&quot;/g;
	return($_);
}

sub http_unquote {
	local($_) = @_;

	s/&quot;/"/g;
	s/&lt;/</g;
	s/&gt;/>/g;
	s/&amp;/&/g;
	return($_);
}

sub strip {
	local($_) = @_;

	s/^\s*//;
	s/\s*$//;
	return($_);
}

sub strip_args {
	local($key);
	local(*args) = @_;

	foreach $key (keys %args) {
		$args{$key} = &strip($args{$key});
	}
}

sub disable_stderr {
	open(STDERR, '>/dev/null');
}

sub get_script_dir {
	$_ = $ENV{'SCRIPT_NAME'};
	s#/[^/]*$##;
	return($_);
}

1;
